@@ -38,9 +38,6 @@ const cosmeticSurveyingMissCountMax =
     parseInt(vAPI.localStorage.getItem('cosmeticSurveyingMissCountMax'), 10) ||
     15;
 
-const COMPILED_SPECIFIC_SECTION = 0;
-const COMPILED_GENERIC_SECTION = 1;
-
 /******************************************************************************/
 /******************************************************************************/
 
@@ -183,13 +180,12 @@ SelectorCacheEntry.junkyard = [];
 //       High-medium generic: [href^="..."]
 //       High-high generic: everything else
 // Specific
-//    Specfic hostname
+//    Specific hostname
 //    Specific entity
 // Generic filters can only be enforced once the main document is loaded.
 // Specific filers can be enforced before the main document is loaded.
 
 const FilterContainer = function() {
-    this.reHasUnicode = /[^\x00-\x7F]/;
     this.rePlainSelector = /^[#.][\w\\-]+/;
     this.rePlainSelectorEscaped = /^[#.](?:\\[0-9A-Fa-f]+ |\\.|\w|-)+/;
     this.rePlainSelectorEx = /^[^#.\[(]+([#.][\w-]+)|([#.][\w-]+)$/;
@@ -398,7 +394,7 @@ FilterContainer.prototype.compileGenericHideSelector = function(
         return;
     }
 
-    writer.select(µb.compiledCosmeticSection + COMPILED_GENERIC_SECTION);
+    writer.select('COSMETIC_FILTERS:GENERIC');
 
     const type = compiled.charCodeAt(0);
     let key;
@@ -432,7 +428,7 @@ FilterContainer.prototype.compileGenericHideSelector = function(
     // https://github.com/uBlockOrigin/uBlock-issues/issues/131
     //   Support generic procedural filters as per advanced settings.
     //   TODO: prevent double compilation.
-    if ( compiled !== raw ) {
+    if ( compiled.charCodeAt(0) === 0x7B /* '{' */ ) {
         if ( µb.hiddenSettings.allowGenericProceduralFilters === true ) {
             return this.compileSpecificSelector(parser, '', false, writer);
         }
@@ -501,7 +497,7 @@ FilterContainer.prototype.compileGenericUnhideSelector = function(
         return;
     }
 
-    writer.select(µb.compiledCosmeticSection + COMPILED_SPECIFIC_SECTION);
+    writer.select('COSMETIC_FILTERS:SPECIFIC');
 
     // https://github.com/chrisaljoudi/uBlock/issues/497
     //   All generic exception filters are stored as hostname-based filter
@@ -531,7 +527,7 @@ FilterContainer.prototype.compileSpecificSelector = function(
         return;
     }
 
-    writer.select(µb.compiledCosmeticSection + COMPILED_SPECIFIC_SECTION);
+    writer.select('COSMETIC_FILTERS:SPECIFIC');
 
     // https://github.com/chrisaljoudi/uBlock/issues/145
     let unhide = exception ? 1 : 0;
@@ -564,13 +560,13 @@ FilterContainer.prototype.compileTemporary = function(parser) {
 
 FilterContainer.prototype.fromCompiledContent = function(reader, options) {
     if ( options.skipCosmetic ) {
-        this.skipCompiledContent(reader, COMPILED_SPECIFIC_SECTION);
-        this.skipCompiledContent(reader, COMPILED_GENERIC_SECTION);
+        this.skipCompiledContent(reader, 'SPECIFIC');
+        this.skipCompiledContent(reader, 'GENERIC');
         return;
     }
 
     // Specific cosmetic filter section
-    reader.select(µb.compiledCosmeticSection + COMPILED_SPECIFIC_SECTION);
+    reader.select('COSMETIC_FILTERS:SPECIFIC');
     while ( reader.next() ) {
         this.acceptedCount += 1;
         const fingerprint = reader.fingerprint();
@@ -606,12 +602,12 @@ FilterContainer.prototype.fromCompiledContent = function(reader, options) {
     }
 
     if ( options.skipGenericCosmetic ) {
-        this.skipCompiledContent(reader, COMPILED_GENERIC_SECTION);
+        this.skipCompiledContent(reader, 'GENERIC');
         return;
     }
 
     // Generic cosmetic filter section
-    reader.select(µb.compiledCosmeticSection + COMPILED_GENERIC_SECTION);
+    reader.select('COSMETIC_FILTERS:GENERIC');
     while ( reader.next() ) {
         this.acceptedCount += 1;
         const fingerprint = reader.fingerprint();
@@ -675,7 +671,7 @@ FilterContainer.prototype.fromCompiledContent = function(reader, options) {
 /******************************************************************************/
 
 FilterContainer.prototype.skipCompiledContent = function(reader, sectionId) {
-    reader.select(µb.compiledCosmeticSection + sectionId);
+    reader.select(`COSMETIC_FILTERS:${sectionId}`);
     while ( reader.next() ) {
         this.acceptedCount += 1;
         this.discardedCount += 1;
@@ -826,6 +822,33 @@ FilterContainer.prototype.getSession = function() {
 
 /******************************************************************************/
 
+FilterContainer.prototype.cssRuleFromProcedural = function(json) {
+    const pfilter = JSON.parse(json);
+    if ( pfilter.cssable !== true ) { return; }
+    const { tasks, action } = pfilter;
+    let mq;
+    if ( tasks !== undefined ) {
+        if ( tasks.length > 1 ) { return; }
+        if ( tasks[0][0] !== 'matches-media' ) { return; }
+        mq = tasks[0][1];
+    }
+    let style;
+    if ( Array.isArray(action) ) {
+        if ( action[0] !== 'style' ) { return; }
+        style = action[1];
+    }
+    if ( mq === undefined && style === undefined ) { return; }
+    if ( mq === undefined ) {
+        return `${pfilter.selector}\n{${style}}`;
+    }
+    if ( style === undefined ) {
+        return `@media ${mq} {\n${pfilter.selector}\n{display:none!important;}\n}`;
+    }
+    return `@media ${mq} {\n${pfilter.selector}\n{${style}}\n}`;
+};
+
+/******************************************************************************/
+
 FilterContainer.prototype.retrieveGenericSelectors = function(request) {
     if ( this.acceptedCount === 0 ) { return; }
     if ( !request.ids && !request.classes ) { return; }
@@ -947,6 +970,8 @@ FilterContainer.prototype.retrieveSpecificSelectors = function(
         domain: request.domain,
         exceptionFilters: [],
         exceptedFilters: [],
+        proceduralFilters: [],
+        convertedProceduralFilters: [],
         noDOMSurveying: this.needDOMSurveyor === false,
     };
     const injectedCSS = [];
@@ -1022,19 +1047,13 @@ FilterContainer.prototype.retrieveSpecificSelectors = function(
         // we extract and inject them immediately.
         if ( proceduralSet.size !== 0 ) {
             for ( const json of proceduralSet ) {
-                const pfilter = JSON.parse(json);
-                if ( pfilter.tasks === undefined ) {
-                    const { action } = pfilter;
-                    if ( action !== undefined && action[0] === ':style' ) {
-                        injectedCSS.push(`${pfilter.selector}\n{${action[1]}}`);
-                        proceduralSet.delete(json);
-                        continue;
-                    }
-                }
-            }
-            if ( proceduralSet.size !== 0 ) {
-                out.proceduralFilters = Array.from(proceduralSet);
+                const cssRule = this.cssRuleFromProcedural(json);
+                if ( cssRule === undefined ) { continue; }
+                injectedCSS.push(cssRule);
+                proceduralSet.delete(json);
+                out.convertedProceduralFilters.push(json);
             }
+            out.proceduralFilters.push(...proceduralSet);
         }
 
         // Highly generic cosmetic filters: sent once along with specific ones.
@@ -1129,6 +1148,37 @@ FilterContainer.prototype.getFilterCount = function() {
 
 /******************************************************************************/
 
+FilterContainer.prototype.dump = function() {
+    let genericCount = 0;
+    for ( const i of [ 'simple', 'complex' ] ) {
+        for ( const j of [ 'id', 'cl' ] ) {
+            genericCount += this.lowlyGeneric[j][i].size;
+        }
+    }
+    return [
+        'Cosmetic Filtering Engine internals:',
+        `specific: ${this.specificFilters.size}`,
+        `generic: ${genericCount}`,
+        `+ lowly.id: ${this.lowlyGeneric.id.simple.size + this.lowlyGeneric.id.complex.size}`,
+        `  + simple: ${this.lowlyGeneric.id.simple.size}`,
+        ...Array.from(this.lowlyGeneric.id.simple).map(a => `    ###${a}`),
+        `  + complex: ${this.lowlyGeneric.id.complex.size}`,
+        ...Array.from(this.lowlyGeneric.id.complex.values()).map(a => `    ##${a}`),
+        `+ lowly.class: ${this.lowlyGeneric.cl.simple.size + this.lowlyGeneric.cl.complex.size}`,
+        `  + simple: ${this.lowlyGeneric.cl.simple.size}`,
+        ...Array.from(this.lowlyGeneric.cl.simple).map(a => `    ##.${a}`),
+        `  + complex: ${this.lowlyGeneric.cl.complex.size}`,
+        ...Array.from(this.lowlyGeneric.cl.complex.values()).map(a => `    ##${a}`),
+        `+ highly: ${this.highlyGeneric.simple.dict.size + this.highlyGeneric.complex.dict.size}`,
+        `  + highly.simple: ${this.highlyGeneric.simple.dict.size}`,
+        ...Array.from(this.highlyGeneric.simple.dict).map(a => `    ##${a}`),
+        `  + highly.complex: ${this.highlyGeneric.complex.dict.size}`,
+        ...Array.from(this.highlyGeneric.complex.dict).map(a => `    ##${a}`),
+    ].join('\n');
+};
+
+/******************************************************************************/
+
 const cosmeticFilteringEngine = new FilterContainer();
 
 export default cosmeticFilteringEngine;
