@@ -24,6 +24,7 @@
 'use strict';
 
 import punycode from '../lib/punycode.js';
+import { i18n$ } from './i18n.js';
 
 /******************************************************************************/
 
@@ -50,8 +51,8 @@ const scopeToSrcHostnameMap = {
     '.': ''
 };
 const hostnameToSortableTokenMap = new Map();
-const statsStr = vAPI.i18n('popupBlockedStats');
-const domainsHitStr = vAPI.i18n('popupHitDomainCount');
+const statsStr = i18n$('popupBlockedStats');
+const domainsHitStr = i18n$('popupHitDomainCount');
 
 let popupData = {};
 let dfPaneBuilt = false;
@@ -643,7 +644,7 @@ const renderTooltips = function(selector) {
         if ( selector !== undefined && key !== selector ) { continue; }
         const elem = uDom.nodeFromSelector(key);
         if ( elem.hasAttribute('title') === false ) { continue; }
-        const text = vAPI.i18n(
+        const text = i18n$(
             details.i18n +
             (uDom.nodeFromSelector(details.state) === null ? '1' : '2')
         );
@@ -840,9 +841,45 @@ const gotoPick = function() {
 /******************************************************************************/
 
 const gotoReport = function() {
+    const popupPanel = {
+        blocked: popupData.pageCounts.blocked.any,
+    };
+    const reportedStates = [
+        { name: 'enabled', prop: 'netFilteringSwitch', expected: true },
+        { name: 'no-cosmetic-filtering', prop: 'noCosmeticFiltering', expected: false },
+        { name: 'no-large-media', prop: 'noLargeMedia', expected: false },
+        { name: 'no-popups', prop: 'noPopups', expected: false },
+        { name: 'no-remote-fonts', prop: 'noRemoteFonts', expected: false },
+        { name: 'no-scripting', prop: 'noScripting', expected: false },
+        { name: 'can-element-picker', prop: 'canElementPicker', expected: true },
+    ];
+    for ( const { name, prop, expected } of reportedStates ) {
+        if ( popupData[prop] === expected ) { continue; }
+        popupPanel[name] = !expected;
+    }
+    if ( hostnameToSortableTokenMap.size !== 0 ) {
+        const blockedDetails = {};
+        const hostnames =
+            Array.from(hostnameToSortableTokenMap.keys()).sort(hostnameCompare);
+        for ( const hostname of hostnames ) {
+            const entry = popupData.hostnameDict[hostname];
+            const count = entry.counts.blocked.any;
+            if ( count === 0 ) { continue; }
+            const domain = entry.domain;
+            if ( blockedDetails[domain] === undefined ) {
+                blockedDetails[domain] = 0;
+            }
+            blockedDetails[domain] += count;
+        }
+        if ( Object.keys(blockedDetails).length !== 0 ) {
+            popupPanel.blockedDetails = blockedDetails;
+        }
+    }
     messaging.send('popupPanel', {
         what: 'launchReporter',
         tabId: popupData.tabId,
+        pageURL: popupData.pageURL,
+        popupPanel: JSON.stringify(popupPanel),
     });
 
     vAPI.closePopup();
@@ -878,8 +915,8 @@ const gotoURL = function(ev) {
 
 /******************************************************************************/
 
-// The popup panel is made of sections. Visiblity of sections can
-// be toggle on/off.
+// The popup panel is made of sections. Visibility of sections can
+// be toggled on/off.
 
 const maxNumberOfSections = 6;
 const sectionFirewallBit = 0b10000;
@@ -1041,6 +1078,7 @@ const reloadTab = function(ev) {
     messaging.send('popupPanel', {
         what: 'reloadTab',
         tabId: popupData.tabId,
+        url: popupData.pageURL,
         select: vAPI.webextFlavor.soup.has('mobile'),
         bypassCache: ev.ctrlKey || ev.metaKey || ev.shiftKey,
     });
@@ -1223,6 +1261,9 @@ const toggleHostnameSwitch = async function(ev) {
 
 */
 
+// https://github.com/uBlockOrigin/uBlock-issues/issues/2145
+//   Ignore events from auto-repeating keys
+
 {
     let eventCount = 0;
     let eventTime = 0;
@@ -1232,6 +1273,7 @@ const toggleHostnameSwitch = async function(ev) {
             eventCount = 0;
             return;
         }
+        if ( ev.repeat ) { return; }
         const now = Date.now();
         if ( (now - eventTime) >= 500 ) {
             eventCount = 0;
@@ -1330,7 +1372,7 @@ const getPopupData = async function(tabId, first = false) {
     };
 
     // The purpose of the following code is to reset to a vertical layout
-    // should the viewport not be enough wide to accomodate the horizontal
+    // should the viewport not be enough wide to accommodate the horizontal
     // layout.
     // To avoid querying a spurious viewport width -- it happens sometimes,
     // somehow -- we delay layout-changing operations to the next paint
