@@ -89,36 +89,34 @@ const contentscriptCode = (( ) => {
     const parts = [
         '(',
         function(hostname, scriptlets) {
+            const doc = document;
             if (
-                document.location === null ||
-                hostname !== document.location.hostname
+                doc.location === null ||
+                hostname !== doc.location.hostname ||
+                typeof self.uBO_scriptletsInjected === 'boolean'
             ) {
                 return;
             }
-            const injectScriptlets = function(d) {
-                let script;
-                try {
-                    script = d.createElement('script');
-                    script.appendChild(d.createTextNode(
-                        decodeURIComponent(scriptlets))
-                    );
-                    (d.head || d.documentElement).appendChild(script);
-                } catch (ex) {
-                }
-                if ( script ) {
-                    if ( script.parentNode ) {
-                        script.parentNode.removeChild(script);
-                    }
-                    script.textContent = '';
-                }
-            };
-            injectScriptlets(document);
+            let script;
+            try {
+                script = doc.createElement('script');
+                script.appendChild(doc.createTextNode(
+                    decodeURIComponent(scriptlets))
+                );
+                (doc.head || doc.documentElement).appendChild(script);
+                self.uBO_scriptletsInjected = true;
+            } catch (ex) {
+            }
+            if ( script ) {
+                script.remove();
+                script.textContent = '';
+            }
+            if ( typeof self.uBO_scriptletsInjected === 'boolean' ) { return 0; }
         }.toString(),
         ')(',
             '"', 'hostname-slot', '", ',
             '"', 'scriptlets-slot', '"',
         ');',
-        '\n0;',
     ];
     return {
         parts: parts,
@@ -431,13 +429,17 @@ scriptletFilteringEngine.injectNow = function(details) {
         matchAboutBlank: true,
         runAt: 'document_start',
     });
-    if ( logEntries === undefined ) { return; }
-    promise.then(results => {
-        if ( Array.isArray(results) === false || results[0] !== 0 ) { return; }
-        for ( const entry of logEntries ) {
-            logOne(entry.tabId, entry.url, entry.token);
-        }
-    });
+    if ( logEntries !== undefined ) {
+        promise.then(results => {
+            if ( Array.isArray(results) === false || results[0] !== 0 ) {
+                return;
+            }
+            for ( const entry of logEntries ) {
+                logOne(entry.tabId, entry.url, entry.token);
+            }
+        });
+    }
+    return scriptlets;
 };
 
 scriptletFilteringEngine.toSelfie = function() {
