@@ -83,17 +83,18 @@ const getElementBoundingClientRect = function(elem) {
     if ( rect.width !== 0 && rect.height !== 0 ) {
         return rect;
     }
+    if ( elem.shadowRoot instanceof DocumentFragment ) {
+        return getElementBoundingClientRect(elem.shadowRoot);
+    }
 
     let left = rect.left,
-        right = rect.right,
+        right = left + rect.width,
         top = rect.top,
-        bottom = rect.bottom;
+        bottom = top + rect.height;
 
     for ( const child of elem.children ) {
         rect = getElementBoundingClientRect(child);
-        if ( rect.width === 0 || rect.height === 0 ) {
-            continue;
-        }
+        if ( rect.width === 0 || rect.height === 0 ) { continue; }
         if ( rect.left < left ) { left = rect.left; }
         if ( rect.right > right ) { right = rect.right; }
         if ( rect.top < top ) { top = rect.top; }
@@ -101,8 +102,10 @@ const getElementBoundingClientRect = function(elem) {
     }
 
     return {
+        bottom,
         height: bottom - top,
         left,
+        right,
         top,
         width: right - left
     };
@@ -218,7 +221,8 @@ const backgroundImageURLFromElement = function(elem) {
 // https://github.com/gorhill/uBlock/issues/1725#issuecomment-226479197
 //   Limit returned string to 1024 characters.
 //   Also, return only URLs which will be seen by an HTTP observer.
-
+// https://github.com/uBlockOrigin/uBlock-issues/issues/2260
+//   Maybe get to the actual URL indirectly.
 const resourceURLsFromElement = function(elem) {
     const urls = [];
     const tagName = elem.localName;
@@ -228,13 +232,15 @@ const resourceURLsFromElement = function(elem) {
         if ( url !== '' ) { urls.push(url); }
         return urls;
     }
-    {
-        const s = elem[prop];
-        if ( typeof s === 'string' && /^https?:\/\//.test(s) ) {
-            urls.push(trimFragmentFromURL(s.slice(0, 1024)));
-        }
+    let s = elem[prop];
+    if ( s instanceof SVGAnimatedString ) {
+        s = s.baseVal;
+    }
+    if ( typeof s === 'string' && /^https?:\/\//.test(s) ) {
+        urls.push(trimFragmentFromURL(s.slice(0, 1024)));
     }
     resourceURLsFromSrcset(elem, urls);
+    resourceURLsFromPicture(elem, urls);
     return urls;
 };
 
@@ -269,6 +275,20 @@ const resourceURLsFromSrcset = function(elem, out) {
     }
 };
 
+// https://github.com/uBlockOrigin/uBlock-issues/issues/2069#issuecomment-1080600661
+// https://developer.mozilla.org/en-US/docs/Web/HTML/Element/picture
+const resourceURLsFromPicture = function(elem, out) {
+    if ( elem.localName === 'source' ) { return; }
+    const picture = elem.parentElement;
+    if ( picture === null || picture.localName !== 'picture' ) { return; }
+    const sources = picture.querySelectorAll(':scope > source');
+    for ( const source of sources ) {
+        const urls = resourceURLsFromElement(source);
+        if ( urls.length === 0 ) { continue; }
+        out.push(...urls);
+    }
+};
+
 /******************************************************************************/
 
 const netFilterFromUnion = function(patternIn, out) {
@@ -333,7 +353,7 @@ const netFilterFromElement = function(elem) {
     const pattern = mergeStrings(urls);
 
 
-    if ( bestCandidateFilter === null ) {
+    if ( bestCandidateFilter === null && elem.matches('html,body') === false ) {
         bestCandidateFilter = {
             type: 'net',
             filters: candidates,
@@ -360,7 +380,9 @@ const netFilter1stSources = {
      'embed': 'src',
     'iframe': 'src',
        'img': 'src',
+     'image': 'href',
     'object': 'data',
+    'source': 'src',
      'video': 'src'
 };
 
@@ -413,6 +435,9 @@ const cosmeticFilterFromElement = function(elem) {
     // Use attributes if still no selector found.
     // https://github.com/gorhill/uBlock/issues/1901
     //   Trim attribute value, this may help in case of malformed HTML.
+    //
+    // https://github.com/uBlockOrigin/uBlock-issues/issues/1923
+    //   Escape unescaped `"` in attribute values
     if ( selector === '' ) {
         let attributes = [], attr;
         switch ( tagName ) {
@@ -454,13 +479,14 @@ const cosmeticFilterFromElement = function(elem) {
         }
         while ( (attr = attributes.pop()) ) {
             if ( attr.v.length === 0 ) { continue; }
+            const w = attr.v.replace(/([^\\])"/g, '$1\\"');
             v = elem.getAttribute(attr.k);
             if ( attr.v === v ) {
-                selector += `[${attr.k}="${attr.v}"]`;
+                selector += `[${attr.k}="${w}"]`;
             } else if ( v.startsWith(attr.v) ) {
-                selector += `[${attr.k}^="${attr.v}"]`;
+                selector += `[${attr.k}^="${w}"]`;
             } else {
-                selector += `[${attr.k}*="${attr.v}"]`;
+                selector += `[${attr.k}*="${w}"]`;
             }
         }
     }
@@ -524,6 +550,22 @@ const filtersFrom = function(x, y) {
         x = undefined;
     }
 
+    // https://github.com/gorhill/uBlock/issues/1545
+    //   Network filter candidates from all other elements found at [x,y].
+    // https://www.reddit.com/r/uBlockOrigin/comments/qmjk36/
+    //   Extract network candidates first.
+    if ( typeof x === 'number' ) {
+        const magicAttr = `${vAPI.sessionId}-clickblind`;
+        pickerRoot.setAttribute(magicAttr, '');
+        const elems = document.elementsFromPoint(x, y);
+        pickerRoot.removeAttribute(magicAttr);
+        for ( const elem of elems ) {
+            netFilterFromElement(elem);
+        }
+    } else if ( first !== null ) {
+        netFilterFromElement(first);
+    }
+
     // Cosmetic filter candidates from ancestors.
     // https://github.com/gorhill/uBlock/issues/2519
     // https://github.com/uBlockOrigin/uBlock-issues/issues/17
@@ -543,16 +585,16 @@ const filtersFrom = function(x, y) {
         }
     }
 
-    // https://github.com/gorhill/uBlock/issues/1545
-    //   Network filter candidates from all other elements found at [x,y].
-    if ( typeof x === 'number' ) {
-        const magicAttr = `${vAPI.sessionId}-clickblind`;
-        pickerRoot.setAttribute(magicAttr, '');
-        const elems = document.elementsFromPoint(x, y);
-        pickerRoot.removeAttribute(magicAttr);
-        for ( const elem of elems ) {
-            netFilterFromElement(elem);
-        }
+    // https://github.com/gorhill/uBlock/commit/ebaa8a8bb28aef043a68c99965fe6c128a3fe5e4#commitcomment-63818019
+    //   If still no best candidate, just use whatever is available in network
+    //   filter candidates -- which may have been previously skipped in favor
+    //   of cosmetic filters.
+    if ( bestCandidateFilter === null && netFilterCandidates.length !== 0 ) {
+        bestCandidateFilter = {
+            type: 'net',
+            filters: netFilterCandidates,
+            slot: 0
+        };
     }
 
     return netFilterCandidates.length + cosmeticFilterCandidates.length;
@@ -634,12 +676,17 @@ const filterToDOMInterface = (( ) => {
         }
 
         // Lookup by tag names.
+        // https://github.com/uBlockOrigin/uBlock-issues/issues/2260
+        //   Maybe get to the actual URL indirectly.
         const elems = document.querySelectorAll(
             Object.keys(netFilter1stSources).join()
         );
         for ( const elem of elems ) {
             const srcProp = netFilter1stSources[elem.localName];
-            const src = elem[srcProp];
+            let src = elem[srcProp];
+            if ( src instanceof SVGAnimatedString ) {
+                src = src.baseVal;
+            }
             if (
                 typeof src === 'string' &&
                     reFilter.test(src) ||
@@ -709,7 +756,7 @@ const filterToDOMInterface = (( ) => {
         try {
             const o = JSON.parse(raw);
             elems = vAPI.domFilterer.createProceduralFilter(o).exec();
-            style = o.action === undefined || o.action[0] !== ':style'
+            style = o.action === undefined || o.action[0] !== 'style'
                 ? vAPI.hideStyle
                 : o.action[1];
         } catch(ex) {
@@ -936,20 +983,22 @@ const zapElementAtPoint = function(mx, my, options) {
 
     if ( elemToRemove instanceof Element === false ) { return; }
 
-    const getStyleValue = function(elem, prop) {
+    const getStyleValue = (elem, prop) => {
         const style = window.getComputedStyle(elem);
         return style ? style[prop] : '';
     };
 
     // Heuristic to detect scroll-locking: remove such lock when detected.
-    let maybeScrollLocked = false;
-    let elem = elemToRemove;
-    do {
-        maybeScrollLocked =
-            parseInt(getStyleValue(elem, 'zIndex'), 10) >= 1000 ||
-            getStyleValue(elem, 'position') === 'fixed';
-        elem = elem.parentElement;
-    } while ( elem !== null && maybeScrollLocked === false );
+    let maybeScrollLocked = elemToRemove.shadowRoot instanceof DocumentFragment;
+    if ( maybeScrollLocked === false ) {
+        let elem = elemToRemove;
+        do {
+            maybeScrollLocked =
+                parseInt(getStyleValue(elem, 'zIndex'), 10) >= 1000 ||
+                getStyleValue(elem, 'position') === 'fixed';
+            elem = elem.parentElement;
+        } while ( elem !== null && maybeScrollLocked === false );
+    }
     if ( maybeScrollLocked ) {
         const doc = document;
         if ( getStyleValue(doc.body, 'overflowY') === 'hidden' ) {
@@ -1224,6 +1273,7 @@ const pickerCSSStyle = [
     'border-radius: 0',
     'box-shadow: none',
     'display: block',
+    'filter: none',
     'height: 100vh',
     'left: 0',
     'margin: 0',
@@ -1237,26 +1287,22 @@ const pickerCSSStyle = [
     'pointer-events: auto',
     'position: fixed',
     'top: 0',
-    'visibility: visible',
+    'transform: none',
+    'visibility: hidden',
     'width: 100%',
     'z-index: 2147483647',
+    'color-scheme: light dark',
     ''
 ];
 
-// https://github.com/uBlockOrigin/uBlock-issues/issues/1408
-//   We need to reset Chromium-specific `color-scheme` property
-//   for our iframe widget.
-if (
-    CSS.supports instanceof Function &&
-    CSS.supports('color-scheme', 'initial')
-) {
-    pickerCSSStyle.push('color-scheme: initial');
-}
 
 const pickerCSS = `
 :root > [${vAPI.sessionId}] {
     ${pickerCSSStyle.join(' !important;')}
 }
+:root > [${vAPI.sessionId}-loaded] {
+    visibility: visible !important;
+}
 :root [${vAPI.sessionId}-clickblind] {
     pointer-events: none !important;
 }
@@ -1279,7 +1325,10 @@ vAPI.MessagingConnection.addListener(onConnectionMessage);
     if ( pickerBootArgs.zap ) {
         url.searchParams.set('zap', '1');
     }
-    pickerRoot.contentWindow.location = url.href;
+    pickerRoot.addEventListener("load", function() {
+      pickerRoot.setAttribute(`${vAPI.sessionId}-loaded`, "");
+    });
+    pickerRoot.src = url.href;
 }
 
 /******************************************************************************/
