@@ -22,6 +22,8 @@
 
 // For background page
 
+/* globals browser */
+
 'use strict';
 
 /******************************************************************************/
@@ -703,7 +705,7 @@ vAPI.setIcon = (( ) => {
             const path = icons[i].path;
             for ( const key in path ) {
                 if ( path.hasOwnProperty(key) === false ) { continue; }
-                imgs.push({ i: i, p: key });
+                imgs.push({ i: i, p: key, cached: false });
             }
         }
 
@@ -721,9 +723,11 @@ vAPI.setIcon = (( ) => {
             for ( const img of imgs ) {
                 if ( img.r.complete === false ) { return; }
             }
-            const ctx = document.createElement('canvas').getContext('2d');
+            const ctx = document.createElement('canvas')
+                .getContext('2d', { willReadFrequently: true });
             const iconData = [ null, null ];
             for ( const img of imgs ) {
+                if ( img.cached ) { continue; }
                 const w = img.r.naturalWidth, h = img.r.naturalHeight;
                 ctx.width = w; ctx.height = h;
                 ctx.clearRect(0, 0, w, h);
@@ -741,6 +745,7 @@ vAPI.setIcon = (( ) => {
                     return;
                 }
                 iconData[img.i][img.p] = imgData;
+                img.cached = true;
             }
             for ( let i = 0; i < iconData.length; i++ ) {
                 if ( iconData[i] ) {
